﻿using System;
using TMPro;
using Unity.VisualScripting;
using UnityEngine;
using UnityEngine.Serialization;

namespace DefaultNamespace
{
    public abstract class Enemy : MonoBehaviour
    {
        [FormerlySerializedAs("_hp")] [SerializeField] protected int hp = 0;
        [FormerlySerializedAs("_damage")] [SerializeField] protected int damage = 0; 
        protected GameDirector gameDirector;
        [FormerlySerializedAs("_sprite")] [SerializeField] protected Sprite sprite;
        protected float Firecool;
        public Vector3 direction = new Vector3(0, 1, 0);
        public float speed = 3f;
        public EnemyProjectile laserenemy; 
        private float _height;
        private float _width;
        private float _ceil;
        private float _floor;
        private bool _up;
        private bool dead = false;
        private AudioSource aS;
        public AudioClip hit;
        public ParticleSystem expl;
        public Transform counter;

        void Start()
        {
            var director = GameObject.Find("GameDirector");
            this.gameDirector = director.GetComponent<GameDirector>();
            this._up = true;
            var cam = Camera.main;
            if (cam != null)
            {
                this._height = 2f * cam.orthographicSize;
                this._width = _height * cam.aspect;
                this._ceil = this._height / 2;
                this._floor = this._ceil * -1;
            }
            InternalStart();
            aS = GetComponent<AudioSource>();
        }

        void Update()
        {
            Move();
            InternalUpdate();
        }

        public abstract void Shoot();
        public abstract void InternalUpdate();
        public abstract void InternalStart();

        public void Spawn(Vector3 pos)
        {
            Instantiate(this.gameObject, pos, Quaternion.identity);
        }
        public void Move()
        {
            Vector3 nextPosition;
            var moveVector = this.direction * this.speed * Time.deltaTime;
            if (this._up)
            {
                nextPosition = this.transform.position + moveVector;
            }
            else
            {
                nextPosition = this.transform.position - moveVector;
            }
            
            if (nextPosition.y > (this._ceil - 0.5) || nextPosition.y < (this._floor + 0.5))
            {
                if (this._up)
                {
                    this._up = false;
                }
                else
                {
                    this._up = true;
                }
            }

            this.transform.position = nextPosition;
        }

        public void Death()
        {
            gameDirector.EnemyDeath();
        }

        public void OnCollisionEnter2D(Collision2D other)
        {
            this.direction *= -1;
        }
        private void OnTriggerEnter2D(Collider2D other) 
        {
            if (other.gameObject.CompareTag("Laser")) 
            {
                int dmg = other.gameObject.GetComponent<Projectile>().GetDmg();
                hp -= dmg;
                var pos = new Vector3(this.transform.position.x, this.transform.position.y, this.transform.position.z);
                var text = Instantiate(this.counter, pos, Quaternion.identity);
                var textPart = text.gameObject.GetComponent<TextMeshPro>();
                textPart.text = dmg.ToString();
                if (hp <= 0 && !dead)
                {
                    dead = true;
                    Death();
                    Instantiate(this.expl, this.transform.position, Quaternion.identity);
                    Destroy(this.gameObject);
                }
                aS.PlayOneShot(hit, 1f);
            }
        }
    }
}